Material resource
{
	MaterialType _type = Opaque;
	VideoProgram _passes
	[ 
		"normal"
		"depth"
		"reflect"
	]
}

VideoProgram normal
{
	VertexLayout _vertexLayout = "MeshLayout\ModelBillboardLayout.rsc";
	VertexProgram _vertexProgram = "vs";
	PixelProgram _pixelProgram = "ps";
	RenderState _renderState = "Material\Opaque\OpaqueMaterial.rsc:rs";
}

VideoProgram depth
{
	VertexLayout _vertexLayout = "MeshLayout\ModelBillboardDepthLayout.rsc";
	VertexProgram _vertexProgram = "vsDepth";
	PixelProgram _pixelProgram = "Material\AlphaTest\AlphaTestMaterial.rsc:psDepth";
	RenderState _renderState = "Material\AlphaTest\AlphaTestMaterial.rsc:rsDepth";
}

VideoProgram reflect
{
	VertexLayout _vertexLayout = "MeshLayout\ModelBillboardLayout.rsc";
	VertexProgram _vertexProgram = "vs";
	PixelProgram _pixelProgram = "ps";
	RenderState _renderState = "Material\Opaque\OpaqueMaterial.rsc:rsRefNone";
}

VertexProgram vs
{
	bool _instanced = true;
	VertexLayout _input = "MeshLayout/ModelBillboardLayout.rsc";
	String _includes
	[
		"Material/Instancing.hlsl"
		"Material/Lighting.hlsl"
	]

	Output _outputs 
	[
		{ String _type = "float4"; String _name = "position";		Usage _usage = Position;			int _index = 0; }
		{ String _type = "float4"; String _name = "texcoord";		Usage _usage = TextureCoordinate;	int _index = 0; }
		{ String _type = "float3"; String _name = "lightFog";		Usage _usage = TextureCoordinate;	int _index = 1; }
		{ String _type = "float3"; String _name = "seasonData";		Usage _usage = TextureCoordinate;	int _index = 2; }
	]

	String _functions
	[
		"
			void Common(float4x4 localToWorld, float4 position, float4 texcoord, float2 userData, float3 normal, out float3 worldPosition, out float4 outputPosition,
				out float4 outputTexcoord, out float3 lightFog, out float3 seasonData)
			{
				position.xyz *= 1.0 / 512.0;
				position.xyz *= localToWorld[3].xyz;
	
				normal = normalize(normal * 2 - 1);
				normal = mul((float3x3)localToWorld, normal).xyz;

				worldPosition = mul((float3x4)localToWorld, float4(position.xyz, 1.0));
				seasonData.xy = worldPosition.xy / 16.0f;
				seasonData.z = gc_userPosition.x * normal.z;
	
				float2 offset = sin(worldPosition.xy + gc_time) * 0.025 * (userData / 8192.0);
				worldPosition.xy += offset;	
		
				outputPosition = mul(gc_worldToProjection, float4(worldPosition, 1.0));
		
				lightFog = GetLightingFog(normal, outputPosition.z);
	
				outputTexcoord = texcoord / float4(512.0, 512.0, 8192.0, 8192.0);
			}
		"
	]

	Variant _variants
	[
		{
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(input.position.w);
				float3 worldPosition;
				Common(localToWorld, input.position, input.texcoord, input.userData, input.normal.xyz, worldPosition, output.position, output.texcoord, output.lightFog, output.seasonData);
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 3; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(input.position.w);
				float3 worldPosition;
				Common(localToWorld, input.position, input.texcoord, input.userData, input.normal.xyz, worldPosition, output.position, output.texcoord, output.lightFog, output.seasonData);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 3; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(input.position.w);
				float3 worldPosition;
				Common(localToWorld, input.position, input.texcoord, input.userData, input.normal.xyz, worldPosition, output.position, output.texcoord, output.lightFog, output.seasonData);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 3; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(input.position.w);
				float3 worldPosition;
				Common(localToWorld, input.position, input.texcoord, input.userData, input.normal.xyz, worldPosition, output.position, output.texcoord, output.lightFog, output.seasonData);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
	]
}

PixelProgram ps
{
	VertexProgram _input = "vs";
	String _includes
	[
		"Material/Texture.hlsl"
		"Material/Shadows.hlsl"
	]

	Texture _textureList
	[
		{ bool _2d = true;	String _name = "diffuse";	int _bindIndex = 0;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "snow";		int _bindIndex = 1;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "aoMap";		int _bindIndex = 3;	bool _exposed = false; }
		{ bool _2d = true;	String _name = "shadowMap";	int _bindIndex = 5;	bool _exposed = false; bool _shadowMap = true; }
	]

	String _functions
	[
		"
			float4 GetColor(float4 texcoord, float3 lightFog, float3 seasonData, float shadow)
			{
				float3 ao = Sample2D(aoMap, aoMapsampler, texcoord.zw).xxx;
				float4 diff = Sample2D(diffuse, diffusesampler, texcoord.xy);
				float3 sno = Sample2D(snow, snowsampler, seasonData.xy);
    
				diff.xyz = lerp(diff.xyz, sno, seasonData.z * ao * ao * ao * ao);
    
				float4 output;
				output.xyz = ComputeLighting(lightFog, diff.xyz, shadow, float3(1, 1, 1), ao);
				output.w = 0;
				clip(diff.w - 0.5);
	
				return output;
			}
		"
	]

	Variant _variants
	[
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, input.seasonData, 1);";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, input.seasonData, GetShadowLow(input.shadowProjection));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, input.seasonData, GetShadowMedium(input.shadowProjection, pc_texelSize.x));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFog, input.seasonData, GetShadowHigh(input.shadowProjection, pc_texelSize.x));";
		}
	]
}

VertexProgram vsDepth
{
	bool _instanced = true;
	VertexLayout _input = "MeshLayout/ModelBillboardDepthLayout.rsc";
	String _includes
	[
		"Material/Instancing.hlsl"
	]

	Output _outputs 
	[
		{ String _type = "float4"; String _name = "position";		Usage _usage = Position;			int _index = 0; }
		{ String _type = "float2"; String _name = "texcoord";		Usage _usage = TextureCoordinate;	int _index = 0; }
	]

	Variant _variants
	[
		{
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(input.position.w);
				
				float3 position = input.position.xyz * localToWorld[3].xyz / 512.0;	
				float3 worldPosition = mul((float3x4)localToWorld, float4(position.xyz, 1.0));
	
				float2 offset = sin(worldPosition.xy + gc_time) * 0.025 * (input.userData / 8192.0f);
				worldPosition.xy += offset;	
	
				output.position = mul(gc_worldToProjection, float4(worldPosition, 1.0));
				output.texcoord = input.texcoord.xy / float2(512.0, 512.0);
			";
		}
	]
}
