Material resource
{
	MaterialType _type = Opaque;
	VideoProgram _passes
	[ 
		"normal"
		"depth"
	]
}

VideoProgram normal
{
	VertexLayout _vertexLayout = "MeshLayout\ModelSkinLayout.rsc";
	VertexProgram _vertexProgram = "vs";
	PixelProgram _pixelProgram = "Material\Opaque\OpaqueMaterial.rsc:ps";
	RenderState _renderState = "Material\Opaque\OpaqueMaterial.rsc:rs";
}

VideoProgram depth
{
	VertexLayout _vertexLayout = "MeshLayout\ModelSkinDepthLayout.rsc";
	VertexProgram _vertexProgram = "vsDepth";
	PixelProgram _pixelProgram = "Material\Opaque\OpaqueMaterial.rsc:psDepth";
	RenderState _renderState = "Material\Opaque\OpaqueMaterial.rsc:rsDepth";
}

VertexProgram vs
{
	bool _instanced = true;
	VertexLayout _input = "MeshLayout/ModelSkinLayout.rsc";
	String _includes
	[
		"Material/Instancing.hlsl"
		"Material/Lighting.hlsl"
		"Material/Skinning.hlsl"
	]

	Output _outputs 
	[
		{ String _type = "float4"; String _name = "position";		Usage _usage = Position;			int _index = 0; }
		{ String _type = "float4"; String _name = "texcoord";		Usage _usage = TextureCoordinate;	int _index = 0; }
		{ String _type = "float3"; String _name = "lightFog";		Usage _usage = TextureCoordinate;	int _index = 1; }
	]

	String _functions
	[
		"
		void Common(float4x4 localToWorld, float4 position, float4 texcoord, float3 normal, float4 index, float4 weight, out float3 worldPosition, out float4 outputPosition,
			out float4 outputTexcoord, out float3 lightFog)
		{
			position.xyz = SkinPosition(position.xyz / 512.0f, index, weight);
			position.xyz *= localToWorld[3].xyz;

			normal = SkinNormal(normal * 2.0 - 1.0, index, weight);
	
			worldPosition = mul((float3x4)localToWorld, float4(position.xyz, 1.0));
			outputPosition = mul(gc_worldToProjection, float4(worldPosition, 1.0));
	
			normal = mul((float3x3)localToWorld, normal).xyz;
	
			lightFog = GetLightingFog(normal, outputPosition.w);
	
			outputTexcoord = texcoord / float4(512.0, 512.0, 8192.0, 8192.0);
		}
		"
	]

	Variant _variants
	[
		{
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(input.position.w);
				float3 worldPosition;
				Common(localToWorld, input.position, input.texcoord, input.normal.xyz, input.index, input.weight, worldPosition, output.position, output.texcoord, output.lightFog);
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 2; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(input.position.w);
				float3 worldPosition;
				Common(localToWorld, input.position, input.texcoord, input.normal.xyz, input.index, input.weight, worldPosition, output.position, output.texcoord, output.lightFog);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 2; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(input.position.w);
				float3 worldPosition;
				Common(localToWorld, input.position, input.texcoord, input.normal.xyz, input.index, input.weight, worldPosition, output.position, output.texcoord, output.lightFog);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 2; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(input.position.w);
				float3 worldPosition;
				Common(localToWorld, input.position, input.texcoord, input.normal.xyz, input.index, input.weight, worldPosition, output.position, output.texcoord, output.lightFog);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
	]
}

VertexProgram vsDepth
{
	bool _instanced = true;
	VertexLayout _input = "MeshLayout/ModelSkinDepthLayout.rsc";
	String _includes
	[
		"Material/Instancing.hlsl"
		"Material/Skinning.hlsl"
	]

	Output _outputs 
	[
		{ String _type = "float4"; String _name = "position";		Usage _usage = Position;			int _index = 0; }
	]

	Variant _variants
	[
		{
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(input.position.w);

				float3 position = input.position.xyz / 512.0;

				position.xyz = SkinPosition(position.xyz, input.index, input.weight);
				position.xyz *= localToWorld[3].xyz;
	
				float3 worldPosition = mul((float3x4)localToWorld, float4(position.xyz, 1.0));
				output.position = mul(gc_worldToProjection, float4(worldPosition, 1.0));
			";
		}
	]
}