Material resource
{
	MaterialType _type = Blended;
	VideoProgram _passes
	[ 
		"normal"
	]
}

VideoProgram normal
{
	VertexLayout _vertexLayout = "MeshLayout\TerrainLayout.rsc";
	VertexProgram _vertexProgram = "vs";
	PixelProgram _pixelProgram = "ps";
	RenderState _renderState = "rs";
}

RenderState rs
{
	AlphaBlend _alphaBlend
	{
		bool _enable = true;
		Blend _source = SourceAlpha;
		Blend _dest = InverseSourceAlpha;
		BlendOp _op = Add;
		ColorWrite _writeMask = Red | Green | Blue | Alpha;
	}
	Depth _depth
	{
		bool _enable = true;
		bool _write = true;
		Compare _compare = LessEqual;
		float _bias = 0.000000;
		float _slopeBias = 0.000000;
	}
	Stencil _stencil
	{
		bool _enable = false;
		bool _twosided = false;
		Compare _compare = Greater;
		uchar _ref = 0;
		StencilOp _fail = Keep;
		StencilOp _zFail = Keep;
		StencilOp _pass = Keep;
		StencilOp _tsFail = Keep;
		StencilOp _tsZFail = Keep;
		StencilOp _tsPass = Keep;
	}
	CullMode _cullMode = None;
	CullMode _fillMode = Solid;
}

VertexProgram vs
{
	VertexLayout _input = "MeshLayout/TerrainLayout.rsc";
	String _includes
	[
		"Material/Instancing.hlsl"
		"Material/Lighting.hlsl"
	]

	Output _outputs 
	[
		{ String _type = "float4"; String _name = "position";			Usage _usage = Position;			int _index = 0; }
		{ String _type = "float4"; String _name = "texcoord";			Usage _usage = TextureCoordinate;	int _index = 0; }
		{ String _type = "float3"; String _name = "viewDir";			Usage _usage = TextureCoordinate;	int _index = 1; }
		{ String _type = "float4"; String _name = "screenProjection";	Usage _usage = TextureCoordinate;	int _index = 2; }
		{ String _type = "float4"; String _name = "lightFogAlpha";		Usage _usage = TextureCoordinate;	int _index = 3; }
	]

	String _functions
	[
		"
			void Common(float4x4 localToWorld, float4 position, float3 normal, float2 texcoord, out float3 worldPosition, out float4 outputPosition,
				out float4 outputTexcoord, out float4 lightFogAlpha, out float4 screenProjection, out float3 viewDir)
			{
				position.z /= 256.0;
				position.xy /= 48.0;
				worldPosition = mul((float3x4)localToWorld, float4(position.xyz, 1.0));
				outputPosition = mul(gc_worldToProjection, float4(worldPosition, 1.0));

				float4x4 so = float4x4(
					float4(0.5, 0.0, 0.0, 0.5),
					float4(0.0, -0.5, 0.0, 0.5),
					float4(0.0, 0.0, 1.0, 0.0),
					float4(0.0, 0.0, 0.0, 1.0));
		
				screenProjection = mul(so, outputPosition);
	
				lightFogAlpha.xyz = GetLightingFog(float3(0, 0, 1), outputPosition.z);
				lightFogAlpha.w = normal.z;
	
				outputTexcoord.xy = texcoord.xy / 1024.0;
				outputTexcoord.zw = texcoord.xy / 1024.0;
				outputTexcoord.w += gc_time * 0.01;
				outputTexcoord.y += gc_time * 0.02 * float2(1.0, 1.2);
	
				viewDir = worldPosition - gc_cameraPosition.xyz;
			}
		"
	]

	Variant _variants
	[
		{
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal.xyz, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha, output.screenProjection, output.viewDir);
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 4; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal.xyz, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha, output.screenProjection, output.viewDir);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 4; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal.xyz, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha, output.screenProjection, output.viewDir);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 4; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal.xyz, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha, output.screenProjection, output.viewDir);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
	]
}

PixelProgram ps
{
	VertexProgram _input = "vs";
	String _includes
	[
		"Material/Texture.hlsl"
		"Material/Shadows.hlsl"
	]

	Texture _textureList
	[
		{ bool _2d = true;	String _name = "water";			int _bindIndex = 0;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "frame";			int _bindIndex = 1;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "reflection";	int _bindIndex = 2;	bool _exposed = true; }
		{ bool _2d = false;	String _name = "highlight";		int _bindIndex = 3;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "shadowMap";		int _bindIndex = 5;	bool _exposed = false; bool _shadowMap = true; }
	]

	String _functions
	[
		"
		float4 GetColor(float4 texcoord, float4 screenProjection, float4 lightFogAlpha, float3 viewDir, float shadow)
		{
			float3 normalA, normalB; 
			normalA.xy = Sample2D(water, watersampler, texcoord.xy).wy * 2 - 1;
			normalA.xy += Sample2D(water, watersampler, texcoord.zw).wy * 2 - 1;
			normalA.xy *= 0.5;
			normalA.z = 1.0 - sqrt(dot(normalA.xy, normalA.xy));

			float3 normal = normalA;
			float2 screenCoord = screenProjection.xy / screenProjection.w;
			float4 frameBuffer = Sample2D(frame, framesampler, (screenCoord + normalA.xy * 0.05));
			
    
			float d = dot(-normalize(viewDir), normal * float3(4, 4, 1));
			
    
			float3 dir = reflect(normalize(viewDir), normal);
			float3 bloom = SampleCUBE(highlight, highlightsampler, dir).xyz * 1.5;

		
			float fresnel = d * d;

			float4 ref = Sample2D(reflection, reflectionsampler, (screenCoord + normalA.xy * 0.6));

			float diffuseAmount = 0.33;
			float refractAmount = (fresnel) * (1 - diffuseAmount);
			float reflectAmount = (1 - fresnel) * (1 - diffuseAmount);
			
			float3 refractColor = frameBuffer.xyz;
			float3 reflectColor = ref.xyz + bloom * ref.a;
			float3 diffuseColor = float3(0.5, 0.55, 0.5) * shadow;
			
			float3 lighting = diffuseColor * diffuseAmount;
			lighting += refractColor * refractAmount;
			lighting += reflectColor * reflectAmount;

			lighting = lerp(lighting.xyz, pc_fogColor.xyz, min(saturate(lightFogAlpha.z), pc_fogColor.w));
			lighting = lerp(frameBuffer.xyz, lighting, frameBuffer.a);

			float4 output;
			output.xyz = lighting;

			output.w = lightFogAlpha.w;

			return output;
		}
		"
	]

	Variant _variants
	[
		{
			String _text = "output.color = GetColor(input.texcoord, input.screenProjection, input.lightFogAlpha, input.viewDir, 1);";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.screenProjection, input.lightFogAlpha, input.viewDir, GetShadowLow(input.shadowProjection));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.screenProjection, input.lightFogAlpha, input.viewDir, GetShadowMedium(input.shadowProjection, pc_texelSize.x));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.screenProjection, input.lightFogAlpha, input.viewDir, GetShadowMedium(input.shadowProjection, pc_texelSize.x));";
		}
	]
}