Material resource
{
	MaterialType _type = Blended;
	VideoProgram _passes
	[
		"normal" 
	]
}

VideoProgram normal
{
	VertexLayout _vertexLayout = "MeshLayout\TerrainDecalLayout.rsc";
	VertexProgram _vertexProgram = "vs";
	PixelProgram _pixelProgram = "ps";
	RenderState _renderState = "rs";
}

RenderState rs
{
	AlphaBlend _alphaBlend
	{
		bool _enable = true;
		Blend _source = SourceAlpha;
		Blend _dest = InverseSourceAlpha;
		BlendOp _op = Add;
		ColorWrite _writeMask = Red | Green | Blue;
	}
	Depth _depth
	{
		bool _enable = true;
		bool _write = false;
		Compare _compare = LessEqual;
		float _bias = -0.0000001;
		float _slopeBias = -0.00001;
	}
	Stencil _stencil
	{
		bool _enable = false;
		bool _twosided = false;
		Compare _compare = Greater;
		uchar _ref = 0;
		StencilOp _fail = Keep;
		StencilOp _zFail = Keep;
		StencilOp _pass = Keep;
		StencilOp _tsFail = Keep;
		StencilOp _tsZFail = Keep;
		StencilOp _tsPass = Keep;
	}
	CullMode _cullMode = Backfaces;
	CullMode _fillMode = Solid;
}


VertexProgram vs
{
	VertexLayout _input = "MeshLayout/TerrainDecalLayout.rsc";
	String _includes
	[
		"Material/Instancing.hlsl"
		"Material/Lighting.hlsl"
		"Material/UnderWater.hlsl"
	]

	Output _outputs 
	[
		{ String _type = "float4"; String _name = "position";		Usage _usage = Position;			int _index = 0; }
		{ String _type = "float3"; String _name = "texcoord";		Usage _usage = TextureCoordinate;	int _index = 0; }
		{ String _type = "float4"; String _name = "lightFogAlpha";	Usage _usage = TextureCoordinate;	int _index = 1; }
	]

	String _functions
	[
		"
			void Common(float4x4 localToWorld, float4 position, float3 normal, float4 texcoord, out float3 worldPosition, out float4 outputPosition,
				out float3 outputTexcoord, out float4 lightFogAlpha)
			{
				position.z /= 256.0;
				worldPosition = mul((float3x4)localToWorld, float4(position.xyz, 1.0));
				outputPosition = mul(gc_worldToProjection, float4(worldPosition, 1.0));
	
				normal = normalize(normal * 2 - 1);
	
				lightFogAlpha.xyz = GetLightingFog(normal, outputPosition.z);
				lightFogAlpha.w = position.w / 255.0f;
	
				outputTexcoord.xy = texcoord.xy / 4096.0;
				outputTexcoord.z = GetUnderWaterAmount(position.z);
			}
		"
	]

	Variant _variants
	[
		{
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha);
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 2; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 2; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
		{
			Output _outputs 
			[
				{ String _type = "float4"; String _name = "shadowProjection";	Usage _usage = TextureCoordinate;	int _index = 2; }
			]
			String _text = "
				float4x4 localToWorld = GetLocalToWorld(0);
				float3 worldPosition;
				Common(localToWorld, input.position, input.normal, input.texcoord, worldPosition, output.position, output.texcoord, output.lightFogAlpha);
				output.shadowProjection = mul(lc_shadowProjection0, float4(worldPosition, 1.0f));
			";
		}
	]
}

PixelProgram ps
{
	VertexProgram _input = "vs";
	String _includes
	[
		"Material/Texture.hlsl"
		"Material/Shadows.hlsl"
	]

	Texture _textureList
	[
		{ bool _2d = true;	String _name = "diffuse";	int _bindIndex = 0;	bool _exposed = true; }
		{ bool _2d = true;	String _name = "shadowMap";	int _bindIndex = 5;	bool _exposed = false; bool _shadowMap = true; }
	]

	String _functions
	[
		"
			float4 GetColor(float3 texcoord, float4 lightFogAlpha, float shadow)
			{
				float4 diff = Sample2D(diffuse, diffusesampler, texcoord.xy);
    
				float4 output;
				output.xyz = ComputeLighting(lightFogAlpha.xyz, diff.xyz, shadow);
				output.xyz = lerp(output.xyz, float3(0.025, 0.123, 0.19), saturate(texcoord.z));
				output.w = diff.w * lightFogAlpha.w;
	
				return output;
			}
		"
	]

	Variant _variants
	[
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFogAlpha, 1);";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFogAlpha, GetShadowLow(input.shadowProjection));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFogAlpha, GetShadowMedium(input.shadowProjection, pc_texelSize.x));";
		}
		{
			String _text = "output.color = GetColor(input.texcoord, input.lightFogAlpha, GetShadowHigh(input.shadowProjection, pc_texelSize.x));";
		}
	]
}
